// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome Buffer */

'use strict';

const BloomFilter = require('jsbloom').filter;
const DDGsettings = require('./settings.es6');
const utils = require('./utils.es6');


class HTTPS {
  constructor() {
    // Store multiple upgrades lists keyed by list name
    this.myUpgradeList = undefined;
    // One whitelist arrray
    this.whitelist = [];
    this.isReady = false;
  }

  // Sets a list by type and name. This is data that
  // is gathered from HTTPSStorage.
  // 'upgrade list' is assumed to be a bloom filter
  // 'whitelist' is an array
  setLists(lists) {
    try {
      lists.forEach((list) => {
        if (!list.data) {
          throw new Error(`HTTPS: ${list.name} missing data`);
        }

        if (list.type === 'upgrade list') {
          const bloom = HTTPS.createBloomFilter(list);
          this.myUpgradeList = bloom;
        } else if (list.type === 'whitelist') {
          this.whitelist = list.data;
        }
      });
      this.isReady = true;
      log('HTTPS: is ready');
    } catch (e) {
      // a failed setLists update will turn https off
      // validation of the data should happen before calling setLists
      this.isReady = false;
      /* eslint-disable-next-line no-console */
      console.log('HTTPS: setLists error, not ready');
      /* eslint-disable-next-line no-console */
      console.log(e);
    }
  }

  // create a new BloomFilter
  // filterData is assumed to be base64 encoded 8 bit typed array
  static createBloomFilter(filterData) {
    const bloom = new BloomFilter(filterData.totalEntries, filterData.errorRate);
    const buffer = Buffer.from(filterData.data, 'base64');
    bloom.importData(buffer);
    return bloom;
  }

  canUpgradeHost(host) {
    if (!this.isReady) {
      /* eslint-disable-next-line no-console */
      console.warn('HTTPS: not ready');
      return false;
    }

    if (this.whitelist.includes(host)) {
      return false;
    }
    const returnVal = this.myUpgradeList.checkEntry(host);
    return returnVal;
  }

  getUpgradedUrl(reqUrl, tab, isMainFrame) {
    if (!this.isReady) {
      /* eslint-disable-next-line no-console */
      console.warn('HTTPS: not ready');
      return reqUrl;
    }

    // Only deal with http calls
    const protocol = utils.getProtocol(reqUrl).toLowerCase();
    if (protocol !== 'http:') {
      return reqUrl;
    }

    // Obey global settings (options page)
    if (!DDGsettings.getSetting('httpsEverywhereEnabled')) {
      return reqUrl;
    }

    // Skip upgrading sites that have been whitelisted by user
    // via on/off toggle in popup
    if (tab.site.whitelisted) {
      log(`HTTPS: ${tab.site.domain} was whitelisted by user. skip upgrade check.`);
      return reqUrl;
    }

    // Determine host without stripping 'www',
    const host = utils.extractHostFromURL(reqUrl, true) || '';

    if (host && this.canUpgradeHost(host)) {
      if (isMainFrame) {
        /* eslint-disable-next-line no-param-reassign */
        tab.mainFrameUpgraded = true;
        HTTPS.incrementUpgradeCount('totalUpgrades');
      }

      return reqUrl.replace(/^(http|https):\/\//i, 'https://');
    }

    // If it falls to here, default to reqUrl
    return reqUrl;
  }

  // Send https upgrade and failure totals
  static sendHttpsUpgradeTotals() {
    const upgrades = DDGsettings.getSetting('totalUpgrades');
    const failed = DDGsettings.getSetting('failedUpgrades');

    // only send if we have data
    if (upgrades || failed) {
      // TODO - add our message
      // pixel.fire('ehs', {'total': upgrades, 'failures': failed})
    }
  }

  // Increment upgrade or failed upgrade settings
  static incrementUpgradeCount(setting) {
    let value = parseInt(DDGsettings.getSetting(setting), 10) || 0;
    value += 1;
    DDGsettings.updateSetting(setting, value);
  }
}

module.exports = new HTTPS();
